# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Class for Malware Image dataset"""
import os
from pathlib import Path
from typing import Any, Tuple

import gdown
import numpy as np
from sklearn.model_selection import train_test_split
import torch
import torchvision


class MalImg(torch.utils.data.Dataset):
    _download_url = "https://drive.google.com/uc?id=1ljOv9NnEsyOPTVC4RFgUGYDy189pNl2S"
    _filename = "malimg_dataset_32x32.npy"

    def __init__(
        self,
        train: bool = True,
        download: bool = True,
        transform: torchvision.transforms = None,
    ):
        """
        Loads the Malware Image dataset.

        Parameters
        ----------
        train: bool
            Whether to load the training split or not.
        download: bool
            Whether to download the dataset or not.
        transform: torchvision.transform
            The transformation pipeline to use for image preprocessing.
        """
        super().__init__()
        self.classes = (
            "Adialer.C",
            "Adialer.C",
            "Adialer.C",
            "Allaple.L",
            "Alueron.gen!J",
            "Autorun.K",
            "C2Lop.P",
            "C2Lop.gen!G",
            "Dialplatform.B",
            "Dontovo.A",
            "Fakerean",
            "Instantaccess",
            "Lolyda.AA 1",
            "Lolyda.AA 2",
            "Lolyda.AA 3",
            "Lolyda.AT",
            "Malex.gen!J",
            "Obfuscator.AD",
            "Rbot!gen",
            "Skintrim.N",
            "Swizzor.gen!E",
            "Swizzor.gen!I",
            "VB.AT",
            "Wintrim.BX",
            "Yuner.A",
        )
        dataset_path = os.path.join(str(Path.home()), "datasets")
        if not os.path.exists(dataset_path):
            os.mkdir(dataset_path)
        if download or not os.path.isfile(os.path.join(dataset_path, MalImg._filename)):
            gdown.download(
                MalImg._download_url,
                os.path.join(dataset_path, MalImg._filename),
                quiet=True,
            )
        dataset = np.load(
            os.path.join(dataset_path, MalImg._filename), allow_pickle=True
        )
        if train:
            train_data, _ = train_test_split(
                dataset, test_size=3e-1, random_state=torch.random.initial_seed()
            )
            features, labels = train_data[:, : (32 ** 2)], train_data[:, -1]
        else:
            _, test_data = train_test_split(
                dataset, test_size=3e-1, random_state=torch.random.initial_seed()
            )
            features, labels = test_data[:, : (32 ** 2)], test_data[:, -1]
        self.data = features.reshape(-1, 32, 32)
        self.targets = labels
        self.transform = transform

    def __getitem__(self, index: int) -> Tuple[Any, Any]:
        features = self.data[index]
        if self.transform:
            features = self.transform(features)
        labels = self.targets[index].astype("int64")
        return (features, labels)

    def __len__(self) -> int:
        return len(self.data)



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""PyTorch dataset loader repository"""
from .create_dataloader import create_dataloader
from .create_dataset import create_dataset
from .encode_features import encode_features
from .load_dataset import load_dataset


__author__ = "Abien Fred Agarap"
__version__ = "0.17.0"


def list_datasets() -> str:
    datasets = [
        "MNIST",
        "Fashion-MNIST",
        "EMNIST",
        "KMNIST",
        "SVHN",
        "CIFAR10",
        "AG News",
        "20 Newsgroups",
        "IMDB",
        "Yelp",
        "Malware Image",
        "Wisconsin Diagnostic Breast Cancer",
        "Binary COVID19",
        "Multi COVID19",
        "USPS",
        "Pima Indians Diabetes",
    ]
    return datasets



from typing import Any, Tuple

from sklearn.datasets import load_breast_cancer
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import torch


class WDBC(torch.utils.data.Dataset):
    def __init__(self, train: bool = True):
        """
        Loads the WDBC dataset.

        Parameter
        ---------
        train: bool
            Whether to load the training set or not.
        """
        super().__init__()
        data = load_breast_cancer()
        self.classes = data.target_names
        features, labels = data.data, data.target
        train_features, test_features, train_labels, test_labels = train_test_split(
            features,
            labels,
            random_state=torch.random.initial_seed(),
            test_size=3e-1,
            shuffle=True,
        )
        scaler = StandardScaler()
        if train:
            train_features = scaler.fit_transform(train_features)
            train_features = train_features.astype("float32")
            self.data = train_features
            self.targets = train_labels
        else:
            test_features = scaler.fit_transform(test_features)
            test_features = test_features.astype("float32")
            self.data = test_features
            self.targets = test_labels

    def __getitem__(self, index: int) -> Tuple[Any, Any]:
        features, labels = self.data[index], self.targets[index]
        return (features, labels)

    def __len__(self):
        return len(self.data)



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Dataset classes"""
from pt_datasets.datasets.AGNews import AGNews
from pt_datasets.datasets.COVID19Dataset import (
    BinaryCOVID19Dataset,
    MultiCOVID19Dataset,
)
from pt_datasets.datasets.IMDB import IMDB
from pt_datasets.datasets.MalImg import MalImg
from pt_datasets.datasets.TwentyNewsgroups import TwentyNewsgroups
from pt_datasets.datasets.WDBC import WDBC
from pt_datasets.datasets.Yelp import Yelp



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Function for encoding features into lower-dimensional space"""
import MulticoreTSNE
import numpy as np
from sklearn.decomposition import PCA

__author__ = "Abien Fred Agarap"


def encode_features(
    features: np.ndarray,
    seed: int = 42,
    dim: int = 30,
    use_cuda: bool = True,
    encoder: str = "pca",
) -> np.ndarray:
    """
    Returns the low-dimensional encoding of the features.

    Parameters
    ----------
    features: np.ndarray
        The input features to encode.
    seed: int
        The random seed to use. Default is [42].
    dim: int
        The dimensionality of the encoded features. Default is [30].
    use_cuda: bool
        Whether to use CUDA for t-SNE encoding or not. Default is [True].
    encoder: str
        The encoder to use. Default is [pca].
        Supported modes: [pca, umap, tsne].

    Returns
    -------
    encoded_features: np.ndarray
        The low-dimensional encoding of the input features.
    """
    supported_encoders = ["pca", "umap", "tsne"]
    assert encoder in supported_encoders, f"Encoder [{encoder}] is not supported."
    if encoder == "pca":
        encoder = PCA(n_components=dim, random_state=seed)
    elif encoder == "tsne" and use_cuda:
        import tsnecuda

        print("[INFO] CUDA-based t-SNE only supports encoding to 2 dimensions.")
        encoder = tsnecuda.TSNE(random_seed=seed)
    elif encoder == "tsne" and not use_cuda:
        encoder = MulticoreTSNE.MulticoreTSNE(
            n_jobs=4, random_state=seed, n_components=dim
        )
    elif encoder == "umap":
        from umap import UMAP

        encoder = UMAP(n_components=dim, random_state=seed)
    encoded_features = encoder.fit_transform(features)
    return encoded_features



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Function for loading datasets"""
from typing import Tuple

from pt_datasets.life import (
    load_binary_covid19,
    load_diabetes,
    load_multi_covid19,
    load_wdbc,
)
from pt_datasets.text import load_dataset as load_text_dataset
from pt_datasets.vision import load_dataset as load_vision_dataset

__author__ = "Abien Fred Agarap"


SUPPORTED_DATASETS = [
    "mnist",
    "fashion_mnist",
    "emnist",
    "cifar10",
    "svhn",
    "malimg",
    "ag_news",
    "20newsgroups",
    "kmnist",
    "imdb",
    "yelp",
    "wdbc",
    "binary_covid",
    "multi_covid",
    "diabetes",
]


def load_dataset(
    name: str = "mnist",
    normalize: bool = True,
    augment: bool = False,
    data_folder: str = "~/datasets",
    vectorizer: str = "tfidf",
    ngram_range: Tuple = (1, 5),
    return_vectorizer: bool = False,
    image_size: int = 64,
    preprocessed_covidx: bool = False,
    preprocessing_bsize: int = 2048,
) -> Tuple[object, object]:
    """
    Returns a tuple of torchvision dataset objects.

    Parameters
    ----------
    name: str
        The name of the dataset to load. Current choices:
            1. mnist (MNIST)
            2. fashion_mnist (FashionMNIST)
            3. emnist (EMNIST/Balanced)
            4. cifar10 (CIFAR10)
            5. svhn (SVHN)
            6. malimg (Malware Image classification)
            7. ag_news (AG News)
            8. 20newsgroups (20 Newsgroups text classification)
            9. kmnist (KMNIST)
            10. wdbc (Wiscosin Diagnostic Breast Cancer classification)
            11. binary_covid (Binary COVID19)
            12. multi_covid (Multi COVID19)
            13. imdb
            14. yelp
            15. usps
            16. diabetes (Pima Indians Diabetes)
    normalize: bool
        Whether to normalize images or not.
    augment: bool
        Whether to perform image augmentation or not.
        This is only used for *MNIST datasets.
    data_folder: str
        The path to the folder for the datasets.
    vectorizer: str
        The vectorization method to use.
        Options: [tfidf (default) | ngrams]
        This is only used for datasets [name = ag_news | 20newsgroups].
    ngram_range: Tuple
        The lower and upper bound of ngram range to use.
        Default: [(3, 3)]
    return_vectorizer: bool
        Whether to return the vectorizer object or not.
        This is only used for datasets [name = ag_news | 20newsgroups].
    image_size: int
        The image size to use for COVID19 datasets.
    preprocessed_covidx: bool
        Whether to use the preprocessed COVID19 datasets or not.
        This requires the use of `modules/export_covid19_dataset`
        in the package repository.
    preprocessing_bsize: int
        The batch size to use for preprocessing the COVID19 dataset.

    Returns
    -------
    Tuple[object, object]
        A tuple consisting of the training dataset and the test dataset.
    """
    supported_datasets = [
        "mnist",
        "fashion_mnist",
        "emnist",
        "cifar10",
        "svhn",
        "malimg",
        "ag_news",
        "20newsgroups",
        "kmnist",
        "wdbc",
        "binary_covid",
        "multi_covid",
        "usps",
        "imdb",
        "yelp",
        "diabetes",
    ]

    name = name.lower()

    _supported = f"Supported datasets: {supported_datasets}"
    assert (
        name in supported_datasets
    ), f"[ERROR] Dataset {name} is not supported. {_supported}"

    if name in [
        "mnist",
        "emnist",
        "fashion_mnist",
        "kmnist",
        "usps",
        "cifar10",
        "svhn",
        "malimg",
    ]:
        train_dataset, test_dataset = load_vision_dataset(
            name=name, data_folder=data_folder, augment=augment, normalize=normalize
        )
    elif name in ["ag_news", "20newsgroups", "imdb", "yelp"]:
        datasets = load_text_dataset(
            name=name,
            vectorizer=vectorizer,
            return_vectorizer=return_vectorizer,
            ngram_range=ngram_range,
        )
        if return_vectorizer:
            train_dataset, test_dataset, vectorizer = datasets
        else:
            train_dataset, test_dataset = datasets
    elif name == "wdbc":
        train_dataset, test_dataset = load_wdbc()
    elif name == "binary_covid":
        train_dataset, test_dataset = load_binary_covid19(
            transform=None,
            size=image_size,
            preprocessed=preprocessed_covidx,
            preprocessing_bsize=preprocessing_bsize,
        )
    elif name == "multi_covid":
        train_dataset, test_dataset = load_multi_covid19(
            transform=None,
            size=image_size,
            preprocessed=preprocessed_covidx,
            preprocessing_bsize=preprocessing_bsize,
        )
    elif name == "diabetes":
        train_dataset, test_dataset = load_diabetes()
    return (
        (train_dataset, test_dataset, vectorizer)
        if return_vectorizer
        else (train_dataset, test_dataset)
    )



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Class for Yelp dataset"""
import csv
import os
from pathlib import Path
from typing import Any, Tuple

from sklearn.model_selection import train_test_split
import torch

from pt_datasets.utils import preprocess_data, vectorize_text


__author__ = "Abien Fred Agarap"


class Yelp(torch.utils.data.Dataset):
    def __init__(
        self,
        train: bool = True,
        vectorizer: str = "tfidf",
        ngram_range: Tuple = (3, 3),
        return_vectorizer: bool = False,
    ):
        """
        Loads the Yelp dataset.

        Parameters
        ----------
        train: bool
            Whether to load the training set or not.
        vectorizer: str
            The vectorizer to use, options: [tfidf (default) | ngrams]
        ngram_range: Tuple
            The lower and upper bound of ngram range to use.
            Default: [(3, 3)]
        return_vectorizer: bool
            Whether to return the vectorizer object or not.
        """
        super().__init__()
        self.classes = ["Negative", "Positive"]
        path = str(Path.home())
        path = os.path.join(path, "datasets/yelp.csv")
        dataset = Yelp.load_data(path)
        (texts, labels) = (list(dataset.keys()), list(dataset.values()))
        train_texts, test_texts, train_labels, test_labels = train_test_split(
            texts,
            labels,
            test_size=0.30,
            random_state=torch.random.initial_seed(),
            shuffle=True,
        )
        if train:
            features, labels = preprocess_data(train_texts, train_labels)
            if return_vectorizer:
                features, vectorizer = vectorize_text(
                    features,
                    vectorizer,
                    return_vectorizer=return_vectorizer,
                    ngram_range=ngram_range,
                )
                self.vectorizer = vectorizer
            else:
                features = vectorize_text(
                    features,
                    vectorizer,
                    return_vectorizer=return_vectorizer,
                    ngram_range=ngram_range,
                )
        else:
            features, labels = preprocess_data(test_texts, test_labels)
            features = vectorize_text(features, vectorizer, ngram_range=ngram_range)
        self.data = features
        self.targets = labels

    def __getitem__(self, index: int) -> Tuple[Any, Any]:
        (text, target) = (self.data[index], self.targets[index])
        target += 1
        target = target.astype("int64")
        return (text, target)

    def __len__(self) -> int:
        return len(self.data)

    @staticmethod
    def load_data(data: str = "~/datasets/yelp.csv"):
        dataset = {}
        with open(data, "r") as file:
            reader = csv.reader(file)
            for index, row in enumerate(reader):
                if index == 0:
                    pass
                else:
                    dataset[row[0]] = int(row[1])
        return dataset



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Medical datasets module"""
import os
from pathlib import Path
from typing import Tuple

import torch
import torchvision

from pt_datasets.datasets.COVID19Dataset import (
    BinaryCOVID19Dataset,
    MultiCOVID19Dataset,
)
from pt_datasets.datasets.Diabetes import Diabetes
from pt_datasets.datasets.WDBC import WDBC
from pt_datasets.download_covid_dataset import (
    download_binary_covid19_dataset,
    download_covidx5_dataset,
)
from pt_datasets.utils import unzip_dataset


def load_wdbc() -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the Wisconsin Diagnostic Breast Cancer dataset.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set for WDBC.
    test_dataset: torch.utils.data.Dataset
        The test set for WDBC.
    """
    train_dataset, test_dataset = WDBC(train=True), WDBC(train=False)
    return train_dataset, test_dataset


def load_binary_covid19(
    transform: torchvision.transforms,
    size: int = 64,
    preprocessed: bool = False,
    preprocessing_bsize: int = 2048,
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Returns a tuple of the tensor datasets for the
    train and test sets of the COVID19 binary classification dataset.

    Parameters
    ----------
    transform: torchvision.transform
        The transformation pipeline to use for image preprocessing.
    size: int
        The size to use for image resizing.
    preprocessed: bool
        Whether to load preprocessed dataset or not.
    preprocessing_bsize: int
        The batch size to use for preprocessing the dataset.

    Returns
    -------
    train_data: torch.utils.data.TensorDataset
        The training set of Binary COVID19 dataset.
    test_data: torch.utils.data.TensorDataset
        The test set of Binary COVID19 dataset.
    """
    dataset_path = os.path.join(str(Path.home()), "datasets")
    if not os.path.exists(dataset_path):
        os.mkdir(dataset_path)
    if not os.path.exists(os.path.join(dataset_path, "BinaryCOVID19Dataset")):
        download_binary_covid19_dataset()
        unzip_dataset(os.path.join(dataset_path, "BinaryCOVID19Dataset.tar.xz"))
    (train_data, test_data) = (
        BinaryCOVID19Dataset(
            train=True,
            preprocessed=preprocessed,
            size=size,
            preprocessing_bsize=preprocessing_bsize,
        ),
        BinaryCOVID19Dataset(
            train=False,
            preprocessed=preprocessed,
            size=size,
            preprocessing_bsize=preprocessing_bsize,
        ),
    )
    return train_data, test_data


def load_multi_covid19(
    transform: torchvision.transforms,
    size: int = 64,
    preprocessed: bool = False,
    preprocessing_bsize: int = 2048,
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Returns a tuple of the tensor datasets for the
    train and test sets of the COVID19 multi-classification dataset.

    Parameters
    ----------
    transform: torchvision.transform
        The transformation pipeline to use for image preprocessing.
    size: int
        The size to use for image resizing.
    preprocessed: bool
        Whether to load preprocessed dataset or not.
    preprocessing_bsize: int
        The batch size to use for preprocessing the dataset.

    Returns
    -------
    train_data: torch.utils.data.TensorDataset
        The training set of COVIDx5 dataset.
    test_data: torch.utils.data.TensorDataset
        The test set of COVIDx5 dataset.
    """
    dataset_path = os.path.join(str(Path.home()), "datasets")
    if not os.path.exists(dataset_path):
        os.mkdir(dataset_path)
    if not os.path.exists(os.path.join(dataset_path, "MultiCOVID19Dataset")):
        download_covidx5_dataset()
        unzip_dataset(os.path.join(dataset_path, "MultiCOVID19Dataset.tar.xz"))
    (train_data, test_data) = (
        MultiCOVID19Dataset(
            train=True,
            preprocessed=preprocessed,
            size=size,
            preprocessing_bsize=preprocessing_bsize,
        ),
        MultiCOVID19Dataset(
            train=False,
            preprocessed=preprocessed,
            size=size,
            preprocessing_bsize=preprocessing_bsize,
        ),
    )
    return train_data, test_data


def load_diabetes() -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the Pima Indians Diabetes dataset.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set.
    """
    train_dataset, test_dataset = Diabetes(train=True), Diabetes(train=False)
    return train_dataset, test_dataset



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Vision datasets module"""
import os
from pathlib import Path
from typing import Tuple

import gdown
import numpy as np
from sklearn.model_selection import train_test_split
import torch
import torchvision


def load_dataset(
    name: str,
    data_folder: str = "~/datasets",
    augment: bool = False,
    normalize: bool = False,
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads an image benchmark classification dataset.

    Parameters
    ----------
    name: str
        The name of the dataset to load.
        The following are the supported datasets:
            1. mnist
            2. emnist
            3. fashion_mnist
            4. kmnist
            5. usps
            6. cifar10
            7. svhn
            8. malimg
    data_folder: str
        The path to the folder for the datasets.
    augment: bool
        Whether to augment the dataset or not.
    normalize: bool
        Whether to normalize the features or not.

    Returns
    -------
    Tuple
        train_dataset: torch.utils.data.Dataset
            The training dataset object.
        test_dataset: torch.utils.data.Dataset
            The test dataset object
    """
    if name == "mnist":
        train_dataset, test_dataset = load_mnist(
            data_folder=data_folder, augment=augment, normalize=normalize
        )
    elif name == "emnist":
        train_dataset, test_dataset = load_emnist(
            data_folder=data_folder, augment=augment
        )
    elif name == "fashion_mnist":
        train_dataset, test_dataset = load_fashion_mnist(
            data_folder=data_folder, augment=augment
        )
    elif name == "kmnist":
        train_dataset, test_dataset = load_kmnist(
            data_folder=data_folder, augment=augment
        )
    elif name == "usps":
        train_dataset, test_dataset = load_usps(data_folder=data_folder)
    elif name == "cifar10":
        train_dataset, test_dataset = load_cifar10(
            data_folder=data_folder, normalize=normalize
        )
    elif name == "svhn":
        train_dataset, test_dataset = load_svhn(data_folder=data_folder)
    elif name == "malimg":
        train_dataset, test_dataset = load_malimg()
    return train_dataset, test_dataset


def load_mnist(
    data_folder: str = "~/datasets", augment: bool = False, normalize: bool = False
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the MNIST training and test datasets.

    Parameters
    ----------
    data_folder: str
        The path to the folder for the datasets.
    augment: bool
        Whether to perform data augmentation or not.
    normalize: bool
        Whether to normalize data or not.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set.
    """
    train_transform = torchvision.transforms.Compose(
        [torchvision.transforms.ToTensor()]
    )
    test_transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    if augment:
        train_transform = torchvision.transforms.Compose(
            [
                torchvision.transforms.ToTensor(),
                torchvision.transforms.RandomHorizontalFlip(),
                torchvision.transforms.RandomVerticalFlip(),
                torchvision.transforms.Normalize((0.1307,), (0.3081,)),
            ]
        )
    elif normalize:
        train_transform = torchvision.transforms.Compose(
            [
                torchvision.transforms.ToTensor(),
                torchvision.transforms.Normalize((0.1307,), (0.3081,)),
            ]
        )
    train_dataset = torchvision.datasets.MNIST(
        root=data_folder, train=True, download=True, transform=train_transform
    )
    test_dataset = torchvision.datasets.MNIST(
        root=data_folder, train=False, download=True, transform=test_transform
    )
    return train_dataset, test_dataset


def load_fashion_mnist(
    data_folder: str = "~/dataset", augment: bool = False
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the Fashion-MNIST training and test datasets.

    Parameters
    ----------
    data_folder: str
        The path to the folder for the datasets.
    augment: bool
        Whether to perform data augmentation or not.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set.
    """
    train_transform = torchvision.transforms.Compose(
        [torchvision.transforms.ToTensor()]
    )
    test_transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    if augment:
        train_transform = torchvision.transforms.Compose(
            [
                torchvision.transforms.ToTensor(),
                torchvision.transforms.RandomHorizontalFlip(),
                torchvision.transforms.RandomVerticalFlip(),
            ]
        )
    train_dataset = torchvision.datasets.FashionMNIST(
        root=data_folder, train=True, download=True, transform=train_transform
    )
    test_dataset = torchvision.datasets.FashionMNIST(
        root=data_folder, train=False, download=True, transform=test_transform
    )
    return train_dataset, test_dataset


def load_emnist(
    data_folder: str = "~/dataset", augment: bool = False
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the EMNIST training and test datasets.


    Parameters
    ----------
    data_folder: str
        The path to the folder for the datasets.
    augment: bool
        Whether to perform data augmentation or not.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set.
    """
    train_transform = torchvision.transforms.Compose(
        [torchvision.transforms.ToTensor()]
    )
    test_transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    if augment:
        train_transform = torchvision.transforms.Compose(
            [
                torchvision.transforms.ToTensor(),
                torchvision.transforms.RandomHorizontalFlip(),
                torchvision.transforms.RandomVerticalFlip(),
            ]
        )
    train_dataset = torchvision.datasets.EMNIST(
        root=data_folder,
        train=True,
        split="balanced",
        download=True,
        transform=train_transform,
    )
    test_dataset = torchvision.datasets.EMNIST(
        root=data_folder,
        train=False,
        split="balanced",
        download=True,
        transform=test_transform,
    )
    return (train_dataset, test_dataset)


def load_kmnist(
    data_folder: str = "~/datasets", augment: bool = False
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the KMNIST training and test datasets.

    Parameter
    ---------
    data_folder: str
        The path to the folder for the datasets.
    normalize: bool
        Whether to normalize the dataset or not.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set.
    """
    train_transform = torchvision.transforms.Compose(
        [torchvision.transforms.ToTensor()]
    )
    test_transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    if augment:
        train_transform = torchvision.transforms.Compose(
            [
                torchvision.transforms.ToTensor(),
                torchvision.transforms.RandomHorizontalFlip(),
                torchvision.transforms.RandomVerticalFlip(),
            ]
        )
    train_dataset = torchvision.datasets.KMNIST(
        root=data_folder, train=True, download=True, transform=train_transform
    )
    test_dataset = torchvision.datasets.KMNIST(
        root=data_folder, train=False, download=True, transform=test_transform
    )
    return (train_dataset, test_dataset)


def load_usps(
    data_folder: str = "~/datasets",
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the USPS training and test datasets.

    Parameter
    ---------
    data_folder: str
        The path to the folder for the datasets.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set.
    """
    train_transform = torchvision.transforms.Compose(
        [torchvision.transforms.ToTensor()]
    )
    test_transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    train_dataset = torchvision.datasets.USPS(
        root=data_folder, train=True, download=True, transform=train_transform
    )
    test_dataset = torchvision.datasets.USPS(
        root=data_folder, train=False, download=True, transform=test_transform
    )
    return (train_dataset, test_dataset)


def load_cifar10(data_folder: str = "~/datasets", normalize: bool = False):
    """
    Loads the CIFAR10 training and test datasets.

    Parameter
    ---------
    data_folder: str
        The path to the folder for the datasets.
    normalize: bool
        Whether to normalize the dataset or not.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set.
    """
    transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    if normalize:
        transform = torchvision.transforms.Compose(
            [
                torchvision.transforms.ToTensor(),
                torchvision.transforms.Normalize(
                    (0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)
                ),
            ]
        )
    train_dataset = torchvision.datasets.CIFAR10(
        root=data_folder, train=True, download=True, transform=transform
    )
    test_dataset = torchvision.datasets.CIFAR10(
        root=data_folder, train=False, download=True, transform=transform
    )
    return train_dataset, test_dataset


def load_svhn(
    data_folder: str = "~/datasets",
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the SVHN training and test datasets.

    Parameter
    ---------
    data_folder: str
        The path to the folder for the datasets.

    Returns
    -------
    Tuple
        train_dataset: torch.utils.data.Dataset
            The training set.
        test_dataset: torch.utils.data.Dataset
            The test set.
    """
    train_transform = torchvision.transforms.Compose(
        [torchvision.transforms.ToTensor()]
    )
    test_transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    train_dataset = torchvision.datasets.SVHN(
        root=data_folder, split="train", download=True, transform=train_transform
    )
    test_dataset = torchvision.datasets.SVHN(
        root=data_folder, split="test", download=True, transform=test_transform
    )
    return train_dataset, test_dataset


def load_malimg(
    test_size: float = 0.3, seed: int = 42
) -> Tuple[torch.utils.data.DataLoader, torch.utils.data.DataLoader]:
    """
    Returns a tuple of tensor datasets for the
    training and test splits of MalImg dataset.

    Parameters
    ----------
    test_size: float
        The size of the test set.
    seed: int
        The random seed to use for splitting.

    Returns
    -------
    Tuple[torch.utils.data.DataLoader, torch.utils.data.DataLoader]
        train_dataset
            The training set of MalImg dataset.
        test_dataset
            The test set of MalImg dataset.
    """
    download_url = "https://drive.google.com/uc?id=1Y6Ha5Jir8EI726KdwAKWHVU-oWmPqNDT"
    malimg_filename = "malimg_dataset_32x32.npy"
    dataset_path = os.path.join(str(Path.home()), "datasets")
    if not os.path.exists(dataset_path):
        os.mkdir(dataset_path)
    if not os.path.isfile(os.path.join(dataset_path, malimg_filename)):
        gdown.download(
            download_url, os.path.join(dataset_path, malimg_filename), quiet=True
        )
    dataset = np.load(os.path.join(dataset_path, malimg_filename), allow_pickle=True)
    train_data, test_data = train_test_split(
        dataset, test_size=test_size, random_state=seed
    )
    train_features, train_labels = train_data[:, : (32 ** 2)], train_data[:, -1]
    test_features, test_labels = test_data[:, : (32 ** 2)], test_data[:, -1]
    train_dataset = torch.utils.data.TensorDataset(
        torch.from_numpy(train_features), torch.from_numpy(train_labels)
    )
    test_dataset = torch.utils.data.TensorDataset(
        torch.from_numpy(test_features), torch.from_numpy(test_labels)
    )
    return train_dataset, test_dataset



import csv
import os
from pathlib import Path
from typing import Any, List, Tuple
import zipfile

import gdown
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import torch


class Diabetes(torch.utils.data.Dataset):
    _path = os.path.join(str(Path.home()), "datasets")
    _dataset = os.path.join(_path, "diabetes.csv")
    _zipped_dataset = os.path.join(_path, "diabetes.zip")
    _url = "https://drive.google.com/uc?id=1kN1FeKs1LXYFFDeItbXN1GCc8buXoGAr"

    def __init__(self, train: bool = True):
        super().__init__()
        if not os.path.isfile(Diabetes._dataset):
            gdown.download(url=Diabetes._url, output=Diabetes._zipped_dataset)
            with zipfile.ZipFile(Diabetes._zipped_dataset, "r") as file:
                file.extractall(Diabetes._path)
        features, labels = Diabetes.load_data(Diabetes._dataset)
        train_features, test_features, train_labels, test_labels = train_test_split(
            features,
            labels,
            test_size=0.30,
            random_state=torch.random.initial_seed(),
            shuffle=True,
        )
        scaler = StandardScaler()
        if train:
            train_features = scaler.fit_transform(train_features)
            train_features = train_features.astype("float32")
            self.data = train_features
            self.targets = train_labels
        else:
            test_features = scaler.fit_transform(test_features)
            test_features = test_features.astype("float32")
            self.data = test_features
            self.targets = test_labels
        self.classes = ["Negative", "Positive"]

    def __getitem__(self, index: int) -> Tuple[Any, Any]:
        features, labels = self.data[index], self.targets[index]
        return (features, labels)

    def __index__(self) -> int:
        return len(self.data)

    @staticmethod
    def load_data(filename: str = "~/datasets/diabetes.csv") -> Tuple[Any, Any]:
        dataset = list()
        with open(filename, "r") as file:
            reader = csv.reader(file)
            for index, row in enumerate(reader):
                if index == 0:
                    continue
                else:
                    row = list(map(lambda element: float(element), row))
                    dataset.append(np.array(row))
        dataset = np.array(dataset)
        features, labels = dataset[:, :-1], dataset[:, -1]
        return features, labels



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Class for 20Newsgroups dataset"""
from typing import Any, Tuple

from sklearn.datasets import fetch_20newsgroups
import torch

from pt_datasets.utils import preprocess_data, vectorize_text

__author__ = "Abien Fred Agarap"


class TwentyNewsgroups(torch.utils.data.Dataset):
    def __init__(
        self,
        train: bool = True,
        vectorizer: str = "tfidf",
        ngram_range: Tuple = (3, 3),
        return_vectorizer: bool = False,
    ):
        """
        Loads the 20 Newsgroups dataset.

        Parameters
        ----------
        train: bool
            Whether to load the training set or not.
        vectorizer: str
            The vectorizer to use, options: [tfidf (default) | ngrams]
        ngram_range: Tuple
            The lower and upper bound of ngram range to use.
            Default: [(3, 3)]
        return_vectorizer: bool
            Whether to return the vectorizer object or not.
        """
        if train:
            self.dataset = fetch_20newsgroups(
                subset="train", remove=("headers", "foooters", "quotes")
            )
            (features, labels) = preprocess_data(self.dataset.data, self.dataset.target)
            if return_vectorizer:
                features, vectorizer = vectorize_text(
                    features,
                    vectorizer=vectorizer,
                    return_vectorizer=return_vectorizer,
                    ngram_range=ngram_range,
                )
                self.vectorizer = vectorizer
            else:
                features = vectorize_text(
                    features, vectorizer=vectorizer, ngram_range=ngram_range
                )
        else:
            self.dataset = fetch_20newsgroups(
                subset="test", remove=("headers", "footers", "quotes")
            )
            (features, labels) = preprocess_data(self.dataset.data, self.dataset.target)
            features = vectorize_text(
                features, vectorizer=vectorizer, ngram_range=ngram_range
            )
        self.classes = self.dataset.target_names
        self.data = features
        self.targets = labels

    def __getitem__(self, index: int) -> Tuple[Any, Any]:
        text, target = self.data[index], self.targets[index]
        target += 1
        target = target.astype("int64")
        return (text, target)

    def __len__(self) -> int:
        return len(self.data)



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""COVID19 dataset classes"""
import os
from pathlib import Path
import pickle
import time
from typing import Dict, List, Tuple

import numpy as np
import torch
import torchvision

from pt_datasets import create_dataloader
from pt_datasets.utils import read_metadata, load_image

__author__ = "Abien Fred Agarap"


DATASET_DIR = os.path.join(str(Path.home()), "datasets")
BINARY_COVID19_DIR = os.path.join(DATASET_DIR, "BinaryCOVID19Dataset")
MULTI_COVID19_DIR = os.path.join(DATASET_DIR, "MultiCOVID19Dataset")
TRAIN_METADATA = "train_split.txt"
TEST_METADATA = "test_split.txt"


class BinaryCOVID19Dataset(torch.utils.data.Dataset):
    """
    Dataset class for the COVID19 binary classification dataset.
    """

    def __init__(
        self,
        train: bool = True,
        transform: torchvision.transforms = None,
        size: int = 64,
        preprocessed: bool = False,
        preprocessing_bsize: int = 2048,
    ):
        """
        Builds the COVID19 binary classification dataset.

        Parameter
        ---------
        train: bool
            Whether to load the training set or not.
        transform: torchvision
            The transformation pipeline to use for image preprocessing.
        size: int
            The size to use for resizing images.
        preprocessed: bool
            Whether to load preprocessed dataset or not.
        preprocessing_bsize: int
            The mini-batch size to use preprocessing the dataset.
        """
        if preprocessed:
            if not os.path.isfile(os.path.join(BINARY_COVID19_DIR, f"train_{size}.pt")):
                print(
                    "[INFO] No preprocessed training dataset found. Preprocessing now..."
                )
                preprocess_dataset(
                    train=True,
                    size=size,
                    export_dir=BINARY_COVID19_DIR,
                    batch_size=preprocessing_bsize,
                )
            if not os.path.isfile(os.path.join(BINARY_COVID19_DIR, f"test_{size}.pt")):
                print("[INFO] No preprocessed test dataset found. Preprocessing now...")
                preprocess_dataset(
                    train=False,
                    size=size,
                    export_dir=BINARY_COVID19_DIR,
                    batch_size=preprocessing_bsize,
                )
            if train:
                if size > 64:
                    dataset = load_pickle(
                        os.path.join(BINARY_COVID19_DIR, f"train_{size}.pt")
                    )
                else:
                    dataset = torch.load(
                        os.path.join(BINARY_COVID19_DIR, f"train_{size}.pt")
                    )
            else:
                if size > 64:
                    dataset = load_pickle(
                        os.path.join(BINARY_COVID19_DIR, f"test_{size}.pt")
                    )
                else:
                    dataset = torch.load(
                        os.path.join(BINARY_COVID19_DIR, f"test_{size}.pt")
                    )
            self.data = dataset[0]
            self.labels = dataset[1]
        else:
            if train:
                path = os.path.join(BINARY_COVID19_DIR, "data/train")
                self.annotations = read_metadata(
                    os.path.join(BINARY_COVID19_DIR, TRAIN_METADATA)
                )
                self.root_dir = path
            else:
                path = os.path.join(BINARY_COVID19_DIR, "data/test")
                self.annotations = read_metadata(
                    os.path.join(BINARY_COVID19_DIR, TEST_METADATA)
                )
                self.root_dir = path
        self.classes = ["negative", "positive"]
        self.transform = transform
        self.size = size
        self.preprocessed = preprocessed

    def __len__(self):
        return len(self.data) if self.preprocessed else len(self.annotations)

    def __getitem__(self, idx) -> Tuple or Dict:
        if torch.is_tensor(idx):
            idx = idx.tolist()
        if self.preprocessed:
            image = self.data[idx]
            if self.transform:
                image = self.transform(image)
            label = self.labels[idx].astype("int64")
            return (image, label)
        else:
            image_name = os.path.join(self.root_dir, self.annotations[idx][1])
            image = load_image(image_name, self.size)
            if self.transform:
                image = self.transform(image)
            label = self.annotations[idx][2]
            label = 0 if label == "negative" else 1
            sample = {"image": image, "label": label}
            return sample


class MultiCOVID19Dataset(torch.utils.data.Dataset):
    """
    Dataset class for the COVID19 multi-classification dataset.
    """

    def __init__(
        self,
        train: bool = True,
        transform: torchvision.transforms = None,
        size: int = 64,
        preprocessed: bool = False,
        preprocessing_bsize: int = 2048,
    ):
        """
        Builds the COVID19 multi-classification dataset.

        Parameter
        ---------
        train: bool
            Whether to load the training set or not.
        transform: torchvision
            The transformation pipeline to use for image preprocessing.
        size: int
            The size to use for resizing images.
        preprocessed: bool
            Whether to load preprocessed dataset or not.
        preprocessing_bsize: int
            The mini-batch size to use preprocessing the dataset.
        """
        if preprocessed:
            if not os.path.isfile(os.path.join(MULTI_COVID19_DIR, f"train_{size}.pt")):
                print(
                    "[INFO] No preprocessed training dataset found. Preprocessing now..."
                )
                preprocess_dataset(
                    train=True,
                    size=size,
                    export_dir=MULTI_COVID19_DIR,
                    batch_size=preprocessing_bsize,
                )
            if not os.path.isfile(os.path.join(MULTI_COVID19_DIR, f"test_{size}.pt")):
                print("[INFO] No preprocessed test dataset found. Preprocessing now...")
                preprocess_dataset(
                    train=False,
                    size=size,
                    export_dir=MULTI_COVID19_DIR,
                    batch_size=preprocessing_bsize,
                )
            if train:
                if size > 64:
                    dataset = load_pickle(
                        os.path.join(MULTI_COVID19_DIR, f"train_{size}.pt")
                    )
                else:
                    dataset = torch.load(
                        os.path.join(MULTI_COVID19_DIR, f"train_{size}.pt")
                    )
            else:
                if size > 64:
                    dataset = load_pickle(
                        os.path.join(MULTI_COVID19_DIR, f"test_{size}.pt")
                    )
                else:
                    dataset = torch.load(
                        os.path.join(MULTI_COVID19_DIR, f"test_{size}.pt")
                    )
            self.data = dataset[0]
            self.labels = dataset[1]
        if train:
            path = os.path.join(MULTI_COVID19_DIR, "data/train")
            self.annotations = read_metadata(
                os.path.join(MULTI_COVID19_DIR, TRAIN_METADATA)
            )
            self.root_dir = path
        else:
            path = os.path.join(MULTI_COVID19_DIR, "data/test")
            self.annotations = read_metadata(
                os.path.join(MULTI_COVID19_DIR, TEST_METADATA)
            )
            self.root_dir = path
        self.classes = ["normal", "non-COVID-19 pneumonia", "COVID-19 pneumonia"]
        self.transform = transform
        self.size = size
        self.preprocessed = preprocessed

    def __len__(self):
        return len(self.data) if self.preprocessed else len(self.annotations)

    def __getitem__(self, idx) -> Dict:
        if torch.is_tensor(idx):
            idx = idx.tolist()
        if self.preprocessed:
            image = self.data[idx]
            if self.transform:
                image = self.transform(image)
            label = self.labels[idx].astype("int64")
            return (image, label)
        else:
            image_name = os.path.join(self.root_dir, self.annotations[idx][1])
            image = load_image(image_name, self.size)
            if self.transform:
                image = self.transform(image)
            label = self.annotations[idx][2]
            if label == "normal":
                label = 0
            elif label == "pneumonia":
                label = 1
            elif label == "COVID-19":
                label = 2
            sample = {"image": image, "label": label}
            return sample


def unpack_examples(data_loader: torch.utils.data.DataLoader) -> Tuple[List, List]:
    """
    Unpacks examples from a data loader,
    and returns them as tuples of list pairs.

    Parameter
    ---------
    data_loader: torch.utils.data.DataLoader
        The data loader object that contains the features-labels pairs.

    Returns
    -------
    Tuple[List, List]
        features: List
            The dataset features.
        labels: List
            The dataset labels.
    """
    features, labels = [], []
    for index, example in enumerate(data_loader):
        start_time = time.time()
        features.append(example.get("image"))
        labels.append(example.get("label"))
        duration = time.time() - start_time
        print(f"[INFO] Processing batch {index + 1} took {duration:.6f}s.")
    return features, labels


def vectorize_examples(
    features: List,
    labels: List,
    dataset_size: int,
    batch_size: int = 512,
    image_size: int = 64,
) -> Tuple[np.ndarray, np.ndarray]:
    """
    Returns a the vectorized features-labels pairs.

    Parameters
    ----------
    features: List
        The dataset features.
    labels: List
        The dataset labels.
    dataset_size: int
        The size of the dataset.
    batch_size: int
        The mini-batch used for unpacking the examples.
    image_size: int
        The number of channels in the dataset features.

    Returns
    -------
    Tuple[np.ndarray, np.ndarray]
        array: np.ndarray
            The vectorized features.
        labels_array: np.ndarray
            The vectorized labels.
    """
    num_channels = 3
    array = np.zeros((dataset_size, num_channels, image_size, image_size))
    labels_array = np.zeros((dataset_size))
    for index, (row, label) in enumerate(zip(features, labels)):
        offset = index * batch_size
        array[offset : offset + batch_size] = row
        labels_array[offset : offset + batch_size] = label
    labels_array = labels_array.astype("int64")
    array = array.astype("float32")
    return array, labels_array


def export_dataset(dataset: Tuple[np.ndarray, np.ndarray], filename: str) -> None:
    """
    Exports the vectorized dataset to a `.pt` file.

    Parameters
    ----------
    dataset: Tuple[np.ndarray, np.ndarray]
        The tuple of vectorized features and labels.
    filename: str
        The path where to save the dataset.
    """
    if not filename.endswith(".pt"):
        filename = f"{filename}.pt"
    if dataset[0].shape[2] > 64:
        with open(filename, "wb") as tensor_file:
            pickle.dump(dataset, tensor_file)
    else:
        torch.save(dataset, filename)


def preprocess_dataset(
    export_dir: str, train: bool = False, size: int = 64, batch_size: int = 2048
) -> None:
    transform = torchvision.transforms.Compose([torchvision.transforms.ToTensor()])
    if train:
        print("[INFO] Loading dataset...")
        if "BinaryCOVID19Dataset" in export_dir:
            train_data = BinaryCOVID19Dataset(
                train=True, size=size, transform=transform
            )
        elif "MultiCOVID19Dataset" in export_dir:
            train_data = MultiCOVID19Dataset(train=True, size=size, transform=transform)
        print("[INFO] Creating data loader...")
        train_loader = create_dataloader(
            train_data, batch_size=batch_size, num_workers=4
        )
        print("[INFO] Unpacking examples...")
        train_features, train_labels = unpack_examples(train_loader)
        print("[INFO] Vectorizing examples...")
        train_features, train_labels = vectorize_examples(
            train_features,
            train_labels,
            dataset_size=len(train_data),
            batch_size=batch_size,
            image_size=size,
        )
        train_dataset = (train_features, train_labels)
        print(
            "[INFO] Exporting dataset to {}".format(
                os.path.join(export_dir, f"train_{size}.pt")
            )
        )
        export_dataset(train_dataset, os.path.join(export_dir, f"train_{size}.pt"))
    else:
        print("[INFO] Loading dataset...")
        if "BinaryCOVID19Dataset" in export_dir:
            test_data = BinaryCOVID19Dataset(
                train=False, size=size, transform=transform
            )
        elif "MultiCOVID19Dataset" in export_dir:
            test_data = MultiCOVID19Dataset(train=False, size=size, transform=transform)
        print("[INFO] Creating data loader...")
        test_loader = create_dataloader(test_data, batch_size=batch_size, num_workers=4)
        print("[INFO] Unpacking examples...")
        test_features, test_labels = unpack_examples(test_loader)
        print("[INFO] Vectorizing examples...")
        test_features, test_labels = vectorize_examples(
            test_features,
            test_labels,
            dataset_size=len(test_data),
            batch_size=batch_size,
            image_size=size,
        )
        test_dataset = (test_features, test_labels)
        print(
            "[INFO] Exporting dataset to {}".format(
                os.path.join(export_dir, f"test_{size}.pt")
            )
        )
        export_dataset(test_dataset, os.path.join(export_dir, f"test_{size}.pt"))


def load_pickle(
    filename: str,
) -> Tuple[np.ndarray or torch.Tensor, np.ndarray or torch.Tensor]:
    """
    Loads the pickled dataset.

    Parameter
    ---------
    filename: str
        The path to the pickled dataset.

    Returns
    -------
    Tuple[np.ndarray or torch.Tensor, np.ndarray or torch.Tensor]
        The first element is the features tensor.
        The second element is the labels tensor.
    """
    with open(filename, "rb") as tensor_file:
        dataset = pickle.load(tensor_file)
    return dataset



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
import csv
import os
from pathlib import Path
import string
import tarfile
from typing import Dict, List, Tuple
from zipfile import ZipFile

import cv2
from imblearn.over_sampling import RandomOverSampler, SMOTE
import nltk
import numpy as np
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
import torch

from . import create_dataset


__author__ = "Abien Fred Agarap"


def read_data(corpus_file: str, label_column: int = 0, document_start: int = 2) -> Dict:
    """
    Returns a <key, value> pair of the loaded dataset
    where the key is the text data and the value is the data label.
    Parameters
    ----------
    corpus_file: str
        The filename of the dataset to load.
    label_column: int
        The column number of the dataset label (zero-indexed).
    document_start: int
        The number of columns in the dataset.
    Returns
    -------
    dataset: Dict
        The <key, value> pair representing the text data and their labels.
    """
    dataset = dict()
    with open(corpus_file, "r", encoding="utf-8") as text_data:
        if corpus_file.endswith(".csv"):
            text_data = csv.reader(text_data, delimiter=",")
            for index, line in enumerate(text_data):
                text = line[-1]
                label = int(line[label_column])
                dataset[text] = label
        else:
            for line in text_data:
                columns = line.strip().split(maxsplit=document_start)
                text = columns[-1]
                label = int(columns[label_column].strip("__label__"))
                dataset[text] = label
    return dataset


def preprocess_data(texts: List, labels: List) -> Tuple[List, np.ndarray]:
    """
    Loads the dataset from file, and returns the processed dataset.

    Parameters
    ----------
    texts: List
        The texts to vectorize.
    labels: List
        The corresponding labels for texts.

    Returns
    -------
    Tuple[List, np.ndarray]
        texts: List
            The preprocessed text features.
        labels: np.ndarray
            The corresponding labels for texts.
    """
    texts = list(
        map(
            lambda text: text.translate(str.maketrans("", "", string.punctuation)),
            texts,
        )
    )
    texts = list(
        map(
            lambda text: " ".join([word for word in text.split() if len(word) > 3]),
            texts,
        )
    )
    texts = list(map(lambda text: text.lower(), texts))
    texts = list(map(lambda text: text.split(), texts))
    en_stopwords = nltk.corpus.stopwords.words("english")
    texts = list(
        map(lambda text: [word for word in text if word not in en_stopwords], texts)
    )
    texts = list(map(lambda text: " ".join(text), texts))
    labels = np.array(labels, dtype=np.float32)
    labels -= 1
    return (texts, labels)


def vectorize_text(
    texts: List,
    vectorizer: str = "tfidf",
    ngram_range: Tuple = (3, 3),
    max_features: int = 2000,
    return_vectorizer: bool = False,
) -> np.ndarray:
    """
    Returns the n-Grams or TF-IDF vector representation of the text.

    Parameters
    ----------
    texts: List
        The texts to vectorize.
    vectorizer: str
        The vectorizer to use.
    ngram_range: Tuple
        The lower and upper boundary of the range
        of n-values for different n-grams to be extracted.
    max_features: int
        The maximum number of features to keep.
    return_vectorizer: bool
        Whether to return the vectorizer object or not.

    Returns
    -------
    vectors: np.ndarray
        The vector representation of the text.
    """
    supported_vectorizers = ["ngrams", "tfidf"]
    assert vectorizer in supported_vectorizers or (
        isinstance(vectorizer, TfidfVectorizer)
        or isinstance(vectorizer, CountVectorizer)
    ), f"{vectorizer} is not supported."

    if vectorizer == "tfidf":
        vectorizer = TfidfVectorizer(
            ngram_range=ngram_range,
            max_features=max_features,
            max_df=0.5,
            smooth_idf=True,
            stop_words="english",
        )
        vectors = vectorizer.fit_transform(texts)
    elif vectorizer == "ngrams":
        vectorizer = CountVectorizer(
            ngram_range=ngram_range,
            max_features=max_features,
            max_df=0.5,
            stop_words="english",
        )
        vectors = vectorizer.fit_transform(texts)
    else:
        print(f"[INFO] Using {vectorizer}")
        vectors = vectorizer.transform(texts)
    vectors = vectors.toarray()
    vectors = vectors.astype(np.float32)
    return (vectors, vectorizer) if return_vectorizer else vectors


def unzip_dataset(dataset_filename: str) -> None:
    """
    Extracts the contents of a `.zip` or `.tar` file.

    Parameter
    ---------
    dataset_filename: str
        The path to the compressed dataset.
    """
    print(f"[INFO] Extracting {dataset_filename}...")
    if dataset_filename.endswith(".zip"):
        with ZipFile(dataset_filename, "r") as zip_object:
            zip_object.extractall(os.path.join(str(Path.home()), "datasets"))
    elif dataset_filename.endswith(".tar.xz"):
        with tarfile.open(dataset_filename, "r") as tar_object:
            tar_object.extractall(os.path.join(str(Path.home()), "datasets"))


def read_metadata(metadata_file: str) -> List:
    """
    Returns a nested list that consists of the following
    metadata for the dataset,
    ID, filename, class, source

    Parameter
    ---------
    metadata_filename: str
        The path to the dataset metadata.

    Returns
    -------
    data: List
        The metadata for the dataset.
    """
    with open(metadata_file, "r") as file:
        data = file.readlines()
    for index in range(len(data)):
        data[index] = data[index].strip("\n").split()
        if data[index][0] == "COVID":
            data[index] = [
                f"{data[index][0]} {data[index][1]}",
                data[index][2],
                data[index][3],
            ]
    return data


def crop_top(image: np.ndarray, percent: float = 8e-2) -> np.ndarray:
    """
    Returns an image whose top has been cropped out.

    Parameters
    ----------
    image: np.ndarray
        The image whose top will be cropped out.
    percent: float
        The percentage of top to crop.

    Returns
    -------
    image: np.ndarray
        The top cropped image.
    """
    offset = int(image.shape[0] * percent)
    return image[offset:]


def central_crop(image: np.ndarray) -> np.ndarray:
    """
    Returns a crop of the image center.
    Code from Wang et al. (2020):
    https://github.com/lindawangg/COVID-Net

    Parameter
    ---------
    image: np.ndarray
        The image whose central crop will be returned.

    Returns
    -------
    image: np.ndarray
        The central cropped image.
    """
    size = min(image.shape[0], image.shape[1])
    offset_height = int((image.shape[0] - size) / 2)
    offset_width = int((image.shape[1] - size) / 2)
    return image[
        offset_height : offset_height + size, offset_width : offset_width + size
    ]


def load_image(filename: str, size: Tuple = 224) -> torch.Tensor:
    """
    Loads the image from file.

    Parameters
    ----------
    filename: str
        The image data to load.
    size: int
        The size to use for the loaded image.

    Returns
    -------
    image: torch.Tensor:
        The loaded image.
    """
    image = cv2.imread(filename)
    image = crop_top(image)
    image = central_crop(image)
    image = cv2.resize(image, (size, size))
    return image


def oversample_dataset(
    features: torch.Tensor, labels: torch.Tensor, seed: int = 42, method: str = "smote"
) -> torch.utils.data.Dataset:
    """
    Oversamples the minority class in the given dataset
    using either synthetic minority oversampling technique
    or random sampling technique

    Parameters
    ----------
    features: torch.Tensor
        The dataset features.
    labels: torch.Tensor
        The dataset labels.
    seed: int
        The random seed to use for sampling.
    method: str
        The method of oversampling to use.

    Returns
    -------
    dataset: torch.utils.data.Dataset
        The dataset that consists of the
        oversampled minority class with other classes.
    """
    supported_modes = ["random", "smote"]
    assert (
        method in supported_modes
    ), f"Supported modes: {supported_modes}, but method = {method}"
    oversampler = (
        SMOTE(random_state=seed, n_jobs=-1)
        if method == "smote"
        else RandomOverSampler(random_state=seed)
    )
    if len(features.shape) > 3:
        input_shape = features.shape
    if len(features.shape) > 2:
        features = features.reshape(features.shape[0], -1)
    features, labels = oversampler.fit_resample(features, labels)
    features = np.reshape(features, (features.shape[0], *input_shape[1:]))
    dataset = create_dataset(features=features, labels=labels)
    return dataset



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Text datasets module"""
from typing import Tuple

import torch

from pt_datasets.datasets.AGNews import AGNews
from pt_datasets.datasets.IMDB import IMDB
from pt_datasets.datasets.TwentyNewsgroups import TwentyNewsgroups
from pt_datasets.datasets.Yelp import Yelp


def load_dataset(
    name: str, vectorizer: str, return_vectorizer: bool, ngram_range: Tuple
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads a text benchmark classification dataset.

    Parameters
    ----------
    name: str
        The name of the dataset to load.
        The following are the supported datasets:
            1. ag_news
            2. 20newsgroups
            3. imdb
            4. yelp
    vectorizer: str
        The vectorizer to use, options: [ngrams | tfidf (default)]
    return_vectorizer: bool
        Whether to return the vectorizer object or not.
    ngram_range: Tuple
        The n-gram range to use.

    Returns
    -------
    Tuple
        train_dataset: torch.utils.data.Dataset
            The training dataset object.
        test_dataset: torch.utils.data.Dataset
            The test dataset object
    """
    if name == "ag_news":
        train_dataset, test_dataset = load_agnews(
            vectorizer=vectorizer,
            return_vectorizer=return_vectorizer,
            ngram_range=ngram_range,
        )
    elif name == "20newsgroups":
        train_dataset, test_dataset = load_20newsgroups(
            vectorizer=vectorizer,
            return_vectorizer=return_vectorizer,
            ngram_range=ngram_range,
        )
    elif name == "yelp":
        train_dataset, test_dataset = load_yelp(
            vectorizer=vectorizer,
            return_vectorizer=return_vectorizer,
            ngram_range=ngram_range,
        )
    elif name == "imdb":
        train_dataset, test_dataset = load_imdb(
            vectorizer=vectorizer,
            return_vectorizer=return_vectorizer,
            ngram_range=ngram_range,
        )
    if return_vectorizer:
        vectorizer = train_dataset.vectorizer
        return train_dataset, test_dataset, vectorizer
    else:
        return train_dataset, test_dataset


def load_agnews(
    vectorizer: str = "tfidf",
    return_vectorizer: bool = False,
    ngram_range: Tuple = (3, 3),
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the AG News dataset.

    Parameters
    ----------
    vectorizer: str
        The vectorizer to use, options: [ngrams | tfidf (default)]
    return_vectorizer: bool
        Whether to return the vectorizer object or not.
    ngram_range: Tuple
        The n-gram range to use.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set
    """
    train_dataset = AGNews(
        train=True,
        vectorizer=vectorizer,
        return_vectorizer=True,
        ngram_range=ngram_range,
    )
    vectorizer = train_dataset.vectorizer
    test_dataset = AGNews(train=False, vectorizer=vectorizer, ngram_range=ngram_range)
    return (train_dataset, test_dataset)


def load_20newsgroups(
    vectorizer: str = "tfidf",
    return_vectorizer: bool = False,
    ngram_range: Tuple = (3, 3),
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the 20newsgroups dataset.

    Parameters
    ----------
    vectorizer: str
        The vectorizer to use, options: [ngrams | tfidf (default)]
    return_vectorizer: bool
        Whether to return the vectorizer object or not.
    ngram_range: Tuple
        The n-gram range to use.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set
    """
    train_dataset = TwentyNewsgroups(
        train=True,
        vectorizer=vectorizer,
        return_vectorizer=True,
        ngram_range=ngram_range,
    )
    vectorizer = train_dataset.vectorizer
    test_dataset = TwentyNewsgroups(
        train=False, vectorizer=vectorizer, ngram_range=ngram_range
    )
    return (train_dataset, test_dataset)


def load_imdb(
    vectorizer: str = "tfidf",
    return_vectorizer: bool = False,
    ngram_range: Tuple = (3, 3),
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the IMDB dataset.

    Parameters
    ----------
    vectorizer: str
        The vectorizer to use, options: [ngrams | tfidf (default)]
    return_vectorizer: bool
        Whether to return the vectorizer object or not.
    ngram_range: Tuple
        The n-gram range to use.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set
    """
    train_dataset = IMDB(
        train=True,
        vectorizer=vectorizer,
        return_vectorizer=True,
        ngram_range=ngram_range,
    )
    vectorizer = train_dataset.vectorizer
    test_dataset = IMDB(vectorizer=vectorizer, train=False, ngram_range=ngram_range)
    return (train_dataset, test_dataset)


def load_yelp(
    vectorizer: str = "tfidf",
    return_vectorizer: bool = False,
    ngram_range: Tuple = (3, 3),
) -> Tuple[torch.utils.data.Dataset, torch.utils.data.Dataset]:
    """
    Loads the Yelp dataset.

    Parameters
    ----------
    vectorizer: str
        The vectorizer to use, options: [ngrams | tfidf (default)]
    return_vectorizer: bool
        Whether to return the vectorizer object or not.
    ngram_range: Tuple
        The n-gram range to use.

    Returns
    -------
    train_dataset: torch.utils.data.Dataset
        The training set.
    test_dataset: torch.utils.data.Dataset
        The test set
    """
    train_dataset = Yelp(
        train=True,
        vectorizer=vectorizer,
        return_vectorizer=True,
        ngram_range=ngram_range,
    )
    vectorizer = train_dataset.vectorizer
    test_dataset = Yelp(vectorizer=vectorizer, train=False, ngram_range=ngram_range)
    return (train_dataset, test_dataset)



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Download functions for COVID19 datasets"""
import os
from pathlib import Path

import gdown

__author__ = "Abien Fred Agarap"


BINARY_COVID19_DATASET = (
    "https://drive.google.com/uc?id=1xfuoZmwjf5wu_H2xgqUevs9rjW1R-Sel"
)
MULTI_COVID19_DATASET = (
    "https://drive.google.com/uc?id=1zjtn3ZRZTIYXMo0NnWNHdOQtpw6IUvUl"
)


def download_binary_covid19_dataset():
    """
    Downloads the COVID19 Binary classification dataset.
    """
    path = os.path.join(str(Path.home()), "datasets")
    filename = "BinaryCOVID19Dataset.tar.xz"
    print("[INFO] Downloading the dataset...")
    gdown.download(BINARY_COVID19_DATASET, os.path.join(path, filename))


def download_covidx5_dataset():
    """
    Downloads the COVID19 multi-classification dataset.
    """
    path = os.path.join(str(Path.home()), "datasets")
    filename = "MultiCOVID19Dataset.tar.xz"
    print("[INFO] Downloading the dataset...")
    gdown.download(MULTI_COVID19_DATASET, os.path.join(path, filename))



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Function for creating Tensor datasets"""
import torch


def create_dataset(
    features: torch.Tensor, labels: torch.Tensor
) -> torch.utils.data.TensorDataset:
    """
    Returns a torch.utils.data.TensorDataset object
    to be used for a data loader object.

    Parameters
    ----------
    features: torch.Tensor
        The input features.
    labels: torch.Tensor
        The input labels.

    Returns
    -------
    dataset: torch.utils.data.TensorDataset
        The Tensor Dataset object.
    """
    if not isinstance(features, torch.Tensor):
        features = torch.from_numpy(features)
    if not isinstance(labels, torch.Tensor):
        labels = torch.from_numpy(labels)
    dataset = torch.utils.data.TensorDataset(features, labels)
    return dataset



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Setup file for pt-datasets"""
import os
from os import path
from pathlib import Path
from setuptools import setup


__author__ = "Abien Fred Agarap"


this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()


def _post_install():
    ag_news_setup = """
        echo "[INFO] Downloading AG News Dataset..."
        mkdir -p ~/datasets
        wget --no-clobber -O ~/datasets/ag_news.train https://raw.githubusercontent.com/AnubhavGupta3377/Text-Classification-Models-Pytorch/master/data/ag_news.train
        wget --no-clobber -O ~/datasets/ag_news.test https://raw.githubusercontent.com/AnubhavGupta3377/Text-Classification-Models-Pytorch/master/data/ag_news.test
        echo "[SUCCESS] Done downloading AG News Dataset."
        """
    os.system(ag_news_setup)


def _download_imdb():
    imdb_dataset = "https://drive.google.com/uc?id=1VhjJUc6hlJfHmEln8b0xfxuzUC36bBHE"
    imdb_path = os.path.join(str(Path.home()), "datasets")
    filename = "imdb.csv"
    if not os.path.isfile(os.path.join(imdb_path, filename)):
        print("[INFO] Downloading the IMDB dataset...")
        imdb_download_script = f"""
        gdown {imdb_dataset} -O {os.path.join(imdb_path, filename)}
        """
        os.system(imdb_download_script)


def _download_yelp():
    yelp_dataset = "https://drive.google.com/uc?id=1-RG97iRFppt3zK939cijAH8JQIhOyqu8"
    yelp_path = os.path.join(str(Path.home()), "datasets")
    filename = "yelp.csv"
    if not os.path.isfile(os.path.join(yelp_path, filename)):
        print("[INFO] Downloading the Yelp dataset...")
        yelp_download_script = f"""
        gdown {yelp_dataset} -O {os.path.join(yelp_path, filename)}
        """
        os.system(yelp_download_script)


def _download_stopwords():
    import nltk

    nltk.download("stopwords")


setup(
    name="pt-datasets",
    version="0.17.0",
    packages=["pt_datasets", "pt_datasets/datasets"],
    url="https://github.com/AFAgarap/pt-datasets",
    license="AGPL-3.0 License",
    author="Abien Fred Agarap",
    author_email="abienfred.agarap@gmail.com",
    description="Library for loading PyTorch datasets and data loaders.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "numpy==1.20.3",
        "torchvision==0.9.1",
        "umap-learn",
        "torch==1.8.1",
        "MulticoreTSNE==0.1",
        "scikit_learn==0.23.2",
        "gdown==3.12.2",
        "nltk==3.5",
        "opencv-python==4.2.0.34",
        "imbalanced_learn==0.7.0",
        "numba",
        "llvmlite",
        "cmake==3.18.4",
    ],
)


_post_install()
_download_imdb()
_download_yelp()
_download_stopwords()



# PyTorch Datasets utility repository
# Copyright (C) 2020  Abien Fred Agarap
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Function for creating a data loader"""
import torch

__author__ = "Abien Fred Agarap"


def create_dataloader(
    dataset: object, batch_size: int = 16, shuffle: bool = True, num_workers: int = 0
) -> object:
    """
    Returns a data loader object, ready to be used by a model.

    Parameters
    ----------
    dataset: object
        The dataset from `torchvision.datasets`.
    batch_size: int
        The mini-batch size for the data loading. Default is [16].
    shuffle: bool
        Whether to shuffle dataset or not. Default is [True].
    num_workers: int
        The number of subprocesses to use for data loading. Default is [0].

    Returns
    -------
    data_loader: torch.utils.dataloader.DataLoader
        The data loader object to be used by a model.
    """
    data_loader = torch.utils.data.DataLoader(
        dataset,
        batch_size=batch_size,
        shuffle=shuffle,
        num_workers=num_workers,
        pin_memory=True,
    )
    return data_loader
